"""Fake SQL utilities for loading CSV data as database-like records.

This module provides functions to load CSV data and return it in formats
that simulate SQL query results for Python 3.8+.
"""

from typing import List, Tuple, Any

from pandas import read_csv


def load_csv_as_records(csv_file_path: str) -> List[Tuple[Any, ...]]:
    """Load CSV data and return as list of tuples (simulating SQL records).

    This function reads a CSV file using pandas and converts each row
    into a tuple, creating a list structure similar to SQL query results.

    Args:
        csv_file_path: Path to the CSV file to be loaded.

    Returns:
        List of tuples where each tuple represents a data row.

    Raises:
        Exception: If CSV loading or processing fails.

    Example:
        >>> records = load_csv_as_records('data/sample.csv')
        >>> records
        [('John', 25, 'Engineer'), ('Jane', 30, 'Manager')]
    """
    try:
        # Load CSV data into pandas DataFrame
        data_frame = read_csv(csv_file_path)

        # Convert DataFrame to list of tuples (simulating SQL records)
        record_list = [tuple(row) for row in data_frame.to_numpy()]

        return record_list

    except Exception as e:
        # Re-raise with more context for debugging
        raise Exception(f'Failed to load CSV file {csv_file_path}: {e}')
