"""Notification utilities for sending messages via various platforms.

This module provides functions for sending notifications through different
services like Telegram for Python 3.8+.
"""

from typing import Dict, Tuple, Union, Any

from requests import post


def send_telegram_message(token: str, message: str, chat_id: str) -> Tuple[bool, Union[Dict[str, Any], str]]:
    """Send a message via Telegram Bot API.

    Args:
        token: Telegram bot token for authentication.
        message: Text message to send (supports HTML formatting).
        chat_id: Target chat ID or username to send message to.

    Returns:
        Tuple containing success status and response data.
        On success: (True, response_dict)
        On failure: (False, error_text)

    Example:
        >>> success, data = send_telegram_message(
        ...     token='your_bot_token',
        ...     message='<b>Hello World!</b>',
        ...     chat_id='@your_channel'
        ... )
    """
    # Build API URL using f-string
    api_url = f'https://api.telegram.org/bot{token}/sendMessage'
    
    # Prepare request payload
    request_payload = {
        "text": message,
        "parse_mode": "html",
        "chat_id": chat_id
    }
    
    # Set appropriate headers for JSON request
    request_headers = {
        "accept": "application/json",
        "content-type": "application/json"
    }
    
    # Send POST request to Telegram API
    response = post(api_url, json=request_payload, headers=request_headers)
    
    # Return success status with appropriate response data
    if response.status_code == 200:
        return True, response.json()
    return False, response.text
