"""Decorator utilities for function timing and performance monitoring.

This module provides decorators for measuring function execution time
for Python 3.8+.
"""

from functools import wraps
from time import time, sleep, strftime, gmtime
from typing import Callable, Any


def measure_execution_time(func: Callable[..., Any]) -> Callable[..., Any]:
    """Decorator to measure and display function execution time.

    This decorator calculates the total execution time of a function
    and prints it in HH:MM:SS format. Includes a 1-second delay
    after function execution for timing accuracy.

    Args:
        func: The function to be decorated.

    Returns:
        Wrapped function that measures execution time.

    Example:
        >>> @measure_execution_time
        ... def slow_function():
        ...     time.sleep(2)
        ...     return "done"
        >>> result = slow_function()  # Prints execution time
    """
    @wraps(func)
    def execution_timer(*args: Any, **kwargs: Any) -> Any:
        """Wrapper function that measures execution time."""
        # Record start time
        start_time = time()
        
        # Execute the original function
        function_result = func(*args, **kwargs)
        
        # Add delay for timing accuracy
        sleep(1)
        
        # Record end time and calculate duration
        end_time = time()
        elapsed_seconds = end_time - start_time
        
        # Format and display execution time
        formatted_time = strftime("%H:%M:%S", gmtime(elapsed_seconds))
        print(f'Execution time for function {func.__name__} is {formatted_time}')
        
        return function_result
    
    return execution_timer
